
local FOLDER_NAME, SSA = ...

local L = LibStub("AceLocale-3.0"):GetLocale("ShamanAuras", true)
local Auras = LibStub("AceAddon-3.0"):GetAddon("ShamanAuras")
local AceGUI = LibStub("AceGUI-3.0");

local lgIcon,lgGlow,smIcon,smGlow = 32,45,25,35;
local x,y = 0,0,0;
local bg,low,med,high = "BACKGROUND","LOW","MEDIUM","HIGH"
local buffTable,mainTable,utilTable = {},{},{};
local mainTotems,EST_Expires,EST_GUID = 0;

local EventFrame = CreateFrame("Frame");
EventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

-- Aura Group Builder
local function CreateGroup(name,parent)
	SSA[name] = CreateFrame("Frame",name,parent);
	local Group = SSA[name];
	Group:SetFrameStrata(bg);
	Group:Show();
	
	return Group;
end

-- Aura Icon Builder
local function InitializeFrames(name,parent,icon,iconSize,glowSize,charge)
	
	SSA[name] = CreateFrame("Frame",name,parent);
	local Frame = SSA[name];
	Frame:SetFrameStrata("LOW");
	Frame:SetWidth(iconSize);
	Frame:SetHeight(iconSize);
	Frame:Show();

	Frame.texture = Frame:CreateTexture(nil,"BACKGROUND");
	Frame.texture:SetTexture("Interface\\addons\\ShamanAuras\\media\\ICONS\\"..icon);
	Frame.texture:SetAllPoints(Frame);
	
	Frame.CD = CreateFrame("Cooldown", name.."CD", Frame, "CooldownFrameTemplate");
	Frame.CD:SetAllPoints(Frame);
	
	Frame.CD.text = Frame.CD:CreateFontString(nil, "MEDIUM", "GameFontHighlightLarge");
	Frame.CD.text:SetAllPoints(Frame.CD);
	Frame.CD.text:SetPoint("CENTER",0,0);
	Frame.CD.text:SetTextColor(1,1,0,1);
	
	if (glowSize) then
		--[[SSA[name.."Glow"] = CreateFrame("Frame",name.."Glow",parent);
		local Glow = SSA[name.."Glow"];
		Glow:SetFrameStrata("BACKGROUND");
		Glow:SetWidth(glowSize);
		Glow:SetHeight(glowSize);]]
		Frame.glow = CreateFrame("Frame",name.."Glow",Frame);
		Frame.glow:SetPoint("CENTER",0,0);
		Frame.glow:SetFrameStrata("BACKGROUND");
		Frame.glow:SetWidth(glowSize);
		Frame.glow:SetHeight(glowSize);
		Frame.glow:Show();
	end
	if (charge) then
		Frame.ChargeCD = CreateFrame("Cooldown", name.."ChargeCD", Frame, "CooldownFrameTemplate");
		Frame.ChargeCD:SetAllPoints(Frame);
		Frame.ChargeCD:SetFrameStrata("LOW");
		Frame.ChargeCD:Show()

		Frame.Charges = CreateFrame("Frame",name.."Charges",Frame);
		Frame.Charges:SetAllPoints(Frame);
		Frame.Charges:SetFrameStrata(med);
		Frame.Charges:SetWidth(iconSize);
		Frame.Charges:SetHeight(iconSize);

		Frame.Charges.text = Frame.Charges:CreateFontString(nil, "MEDIUM", "GameFontHighlightLarge");
		Frame.Charges.text:SetPoint("BOTTOMRIGHT",-3,3);
		Frame.Charges.text:SetFont("Interface\\addons\\ShamanAuras\\Media\\fonts\\ABF.ttf", 13.5,"OUTLINE");
		Frame.Charges.text:SetTextColor(1,1,1,1);		
	end
	_G["SSA_"..name] = Frame;
end

-----------------------------------------------------------------------
-- Restoration Frames
-----------------------------------------------------------------------
local AuraGroup,LargeIconGrpTop,LargeIconGrpBot,SmallIconGrpLeft,SmallIconGrpRight

-- Build Restoration Aura Group Containers
AuraGroup = CreateGroup("AuraGroupRes",UIParent);
LargeIconGrpTop = CreateGroup("LargeIconGrpTopRes",AuraGroup);
LargeIconGrpBot = CreateGroup("LargeIconGrpBotRes",AuraGroup);
SmallIconGrpLeft = CreateGroup("SmallIconGrpLeftRes",AuraGroup);
SmallIconGrpRight = CreateGroup("SmallIconGrpRightRes",AuraGroup);

-- Build Large Restoration Icon Frames
InitializeFrames("AscendanceRes",LargeIconGrpBot,"shared\\ascendance",lgIcon);
InitializeFrames("CloudburstTotem",LargeIconGrpTop,"totems\\cloudburst_totem",lgIcon);
InitializeFrames("GiftOfQueen",LargeIconGrpBot,"restoration\\gift_of_the_queen",lgIcon);
InitializeFrames("HealingRain",LargeIconGrpTop,"shared\\healing_rain",lgIcon);
InitializeFrames("HealingStreamTotem",LargeIconGrpTop,"totems\\healing_stream_totem",lgIcon,nil,true);
InitializeFrames("HealingTideTotem",LargeIconGrpBot,"totems\\healing_tide_totem",lgIcon);
InitializeFrames("Riptide",LargeIconGrpTop,"restoration\\riptide",lgIcon,nil,true);
InitializeFrames("SpiritLinkTotem",LargeIconGrpBot,"totems\\spirit_link_totem",lgIcon);
InitializeFrames("UnleashLife",LargeIconGrpTop,"restoration\\unleash_life",lgIcon);
InitializeFrames("Wellspring",LargeIconGrpBot,"restoration\\wellspring",lgIcon);
InitializeFrames("WindRushTotemRes",LargeIconGrpBot,"totems\\wind_rush_totem",lgIcon);

-- Build Small Restoration Icon Frames
InitializeFrames("AncestralGuidanceRes",SmallIconGrpRight,"shared\\ancestral_guidance",smIcon);
InitializeFrames("AstralShiftRes",SmallIconGrpRight,"shared\\astral_shift",smIcon);
InitializeFrames("EarthenShieldTotem",SmallIconGrpLeft,"totems\\earthen_shield_totem",smIcon);
InitializeFrames("EarthgrabTotemRes",SmallIconGrpRight,"totems\\earthgrab_totem",smIcon);
InitializeFrames("FlameShockRes",SmallIconGrpRight,"shared\\flame_shock",smIcon,smGlow);
InitializeFrames("GustWindRes",SmallIconGrpRight,"shared\\gust_of_wind",smIcon);
InitializeFrames("HexRes",SmallIconGrpLeft,"shared\\hex",smIcon);
InitializeFrames("LavaBurstRes",SmallIconGrpRight,"elemental\\lava_burst",smIcon,smGlow,true);
InitializeFrames("LightningSurgeTotemRes",SmallIconGrpRight,"totems\\lightning_surge_totem",smIcon);
InitializeFrames("PurifySpirit",SmallIconGrpLeft,"restoration\\purify_spirit",smIcon);
InitializeFrames("SpiritwalkersGrace",SmallIconGrpLeft,"restoration\\spiritwalkers_grace",smIcon);
InitializeFrames("VoodooTotemRes",SmallIconGrpLeft,"totems\\voodoo_totem",smIcon);
InitializeFrames("WindShearRes",SmallIconGrpLeft,"shared\\wind_shear",smIcon);

-------------------------------------------------------------------------------------------------------
----- Initialize Scripts
-------------------------------------------------------------------------------------------------------

-- Flame Shock
SSA.FlameShockRes:SetScript("OnUpdate", function(self)
	if (Auras:CharacterCheck(3)) then
		local debuff,_,_,_,_,duration,expires,caster = UnitDebuff('target',Auras:GetSpellName(188838));
		local start,duration = GetSpellCooldown(Auras:GetSpellName(188838));
		
		if (not self.text) then
			self.text = self:CreateFontString(nil, "HIGH", "GameFontHighlightLarge");
			self.text:SetPoint("CENTER",0.5,0.5);
			self.text:SetFont("Interface\\addons\\ShamanAuras\\media\\fonts\\PT_Sans_Narrow.TTF", 16,"OUTLINE");
			self.text:SetTextColor(1,1,0,1);
		end
	
		Auras:SpellRangeCheck(self,188838,true);	
		
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true,true);
		end
		
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			if (debuff and caster == 'player') then
				local timer,seconds = Auras:parseTime(expires - GetTime(),false);
				self.text:SetText(timer);
				
				if (seconds <= Auras.db.char.triggers.res.flameShock and UnitAffectingCombat('player')) then
					Auras:ToggleOverlayGlow(self.glow,true,true);
					--isGlowActive = true;
				elseif (seconds == 0) then
					self.text:SetText('');
				else
					Auras:ToggleOverlayGlow(self.glow,false);
					--isGlowActive = false;
				end
				--[[if (seconds == 0) then
					self.text:SetText('');
				else
					self.text:SetText(timer);
				end]]
			else
				self.text:SetText('');
				if (Auras:IsTargetEnemy()) then
					Auras:ToggleOverlayGlow(self.glow,true,true);
				else
					Auras:ToggleOverlayGlow(self.glow,false);
				end
			end
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.ele.OoCAlpha)
		end
	end
end);

-- Lava Burst
SSA.LavaBurstRes:SetScript("OnUpdate", function(self)
	if (Auras:CharacterCheck(3)) then
		local buff = UnitBuff('player',Auras:GetSpellName(77762));
		local start,duration = GetSpellCooldown(Auras:GetSpellName(51505))
		local charges,maxCharges,chgStart,chgDuration = GetSpellCharges(51505)
		local _,_,_,selected = GetTalentInfo(6,3,1);
		
		Auras:SpellRangeCheck(self,51505,true);	
		
		if (selected) then
			if (charges == 2) then
				self.ChargeCD:Hide();
				self.ChargeCD:SetCooldown(0,0);
				self.Charges.text:SetText(2);
			elseif (charges < 2) then
				self.ChargeCD:Show();
				self.ChargeCD:SetCooldown(chgStart,chgDuration);
				self.Charges.text:SetText(charges);
			end
			if (charges > 0) then
				self.Charges.text:SetText(charges);
				self.CD.text:SetText('');
				--self.ChargeCD:Show();
			else
				Auras:ExecuteCooldown(self,chgStart,chgDuration,false);
				self.Charges.text:SetText('');
				self.ChargeCD:Hide();
			end
		else
			self.ChargeCD:Hide();
		end
		
		if (UnitAffectingCombat('player')) then
			if (duration > 2) then
				Auras:ToggleCooldownSwipe(self.CD,true)
				Auras:ToggleOverlayGlow(self.glow,false);
				Auras:ExecuteCooldown(self,start,duration,false);
				self.CD:Show();
			elseif (buff) then
				Auras:ToggleCooldownSwipe(self.CD,false)
				Auras:ToggleOverlayGlow(self.glow,true,false);
				self.CD:Hide();
				self.CD:SetCooldown(0,0);
				Auras:ToggleOverlayGlow(self.glow,true,false);
			else
				Auras:ToggleCooldownSwipe(self.CD,false)
				Auras:ToggleOverlayGlow(self.glow,false);
				self.CD:Hide();
			end
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			if (buff) then
				Auras:ToggleOverlayGlow(self.glow,false);
			end
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Ancestral Guidance
SSA.AncestralGuidanceRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(108281));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Ascendance
SSA.AscendanceRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(114052));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Astral Shift
SSA.AstralShiftRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(108271));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Cloudburst Totem
SSA.CloudburstTotem:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(157153));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Earthen Shield Totem
SSA.EarthenShieldTotem:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(198838));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Earthgrab Totem
SSA.EarthgrabTotemRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(51485));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Gift of the Queen
SSA.GiftOfQueen:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(207778));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Gust of Wind
SSA.GustWindRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(192063));
		
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Healing Rain
SSA.HealingRain:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(73920));
		
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Healing Stream Totem
SSA.HealingStreamTotem:SetScript("OnUpdate", function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(5394))
		local charges,maxCharges,chgStart,chgDuration = GetSpellCharges(5394)
		local _,_,_,selected = GetTalentInfo(6,3,1);

		if (selected) then
			if (charges == 2) then
				self.ChargeCD:Hide();
				self.ChargeCD:SetCooldown(0,0);
			elseif (charges < 2) then
				self.ChargeCD:Show();
				self.ChargeCD:SetCooldown(chgStart,chgDuration);
			end
			if (charges > 0) then
				self.Charges.text:SetText(charges);
				self.CD.text:SetText('');
			else
				Auras:ExecuteCooldown(self,chgStart,chgDuration,false);
				self.Charges.text:SetText('');
			end
		else
			self.Charges.text:SetText('');
			if (duration > 2 and not buff) then
				Auras:ToggleCooldownSwipe(self.CD,true)
				Auras:ExecuteCooldown(self,start,duration,false);
				self.CD:Show();
			else
				Auras:ToggleCooldownSwipe(self.CD,true)
				self.CD:Hide();
			end
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Healing Tide Totem
SSA.HealingTideTotem:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(108280));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Hex
SSA.HexRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(51514));
		
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
		
		Auras:SpellRangeCheck(self,51514,(UnitCreatureType('target') == "Humanoid" or UnitCreatureType('target') == "Beast" or UnitCreatureType('target') == "Critter"));	
		
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Lightning Surge Totem
SSA.LightningSurgeTotemRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(192058));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Purify Spirit
SSA.PurifySpirit:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(77130));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Riptide
SSA.Riptide:SetScript("OnUpdate", function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(61295))
		local charges,maxCharges,chgStart,chgDuration = GetSpellCharges(61295)
		local _,_,_,selected = GetTalentInfo(6,3,1);
	
		if (selected) then
			if (charges == 2) then
				self.ChargeCD:Hide();
				self.ChargeCD:SetCooldown(0,0);
			elseif (charges < 2) then
				self.ChargeCD:Show();
				self.ChargeCD:SetCooldown(chgStart,chgDuration);
			end
			if (charges > 0) then
				self.Charges.text:SetText(charges);
				self.CD.text:SetText('');
			else
				Auras:ExecuteCooldown(self,chgStart,chgDuration,false);
				self.Charges.text:SetText('');
			end
		else
			self.Charges.text:SetText('');
			if (duration > 2 and not buff) then
				Auras:ToggleCooldownSwipe(self.CD,true)
				Auras:ExecuteCooldown(self,start,duration,false);
				self.CD:Show();
			else
				Auras:ToggleCooldownSwipe(self.CD,true)
				self.CD:Hide();
			end
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Spirit Link Totem
SSA.SpiritLinkTotem:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(98008));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Spiritwalker's Grace
SSA.SpiritwalkersGrace:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(79206));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Unleash Life
SSA.UnleashLife:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(73685));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Voodoo Totem
SSA.VoodooTotemRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(196932));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Wellspring
SSA.Wellspring:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(197995));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,false);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Wind Shear
SSA.WindShearRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(57994));
	
		Auras:SpellRangeCheck(self,57994,true);
		
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-- Wind Rush Totem
SSA.WindRushTotemRes:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local start,duration = GetSpellCooldown(Auras:GetSpellName(192077));
	
		if ((duration or 0) > 2) then
			Auras:ExecuteCooldown(self,start,duration,true);
			self.CD:Show();
		else
			self.CD:Hide();
		end
			
		if (UnitAffectingCombat('player') and Auras:IsTargetEnemy()) then
			self:SetAlpha(1);
		else
			self:SetAlpha(Auras.db.char.triggers.res.OoCAlpha)
		end
	end
end);

-------------------------------------------------------------------------------------------------------
----- Build and Initialize Status Bars
-------------------------------------------------------------------------------------------------------

-- Build Tidal Waves Status Bar
local TidalWavesBar = CreateFrame("StatusBar","TidalWavesBar",AuraGroup);
TidalWavesBar:SetStatusBarTexture("Interface\\addons\\ShamanAuras\\media\\statusbar\\halvess");
TidalWavesBar:GetStatusBarTexture():SetHorizTile(false);
TidalWavesBar:GetStatusBarTexture():SetVertTile(false);
TidalWavesBar:SetPoint("CENTER",AuraGroup,"CENTER",0,-202);
--TidalWavesBar:SetWidth(225);
--TidalWavesBar:SetHeight(7);
TidalWavesBar:SetMinMaxValues(0,2);
TidalWavesBar:SetFrameStrata("LOW");
TidalWavesBar:SetStatusBarColor(0.35,0.76,1);
TidalWavesBar:SetAlpha(0);
TidalWavesBar:Show();

TidalWavesBar.bg = TidalWavesBar:CreateTexture(nil,"BACKGROUND");
TidalWavesBar.bg:SetTexture("Interface\\TargetingFrame\\UI-StatusBar");
TidalWavesBar.bg:SetAllPoints(true);
TidalWavesBar.bg:SetVertexColor(0,0,0);
TidalWavesBar.bg:SetAlpha(0.65);

-- Animation for Tidal Waves Status Bar
TidalWavesBar.Flash = TidalWavesBar:CreateAnimationGroup();
TidalWavesBar.Flash:SetLooping("BOUNCE");

TidalWavesBar.Flash.fadeIn = TidalWavesBar.Flash:CreateAnimation("Alpha");
TidalWavesBar.Flash.fadeIn:SetFromAlpha(1);
TidalWavesBar.Flash.fadeIn:SetToAlpha(0.25);
TidalWavesBar.Flash.fadeIn:SetDuration(0.33);

TidalWavesBar.Flash.fadeOut = TidalWavesBar.Flash:CreateAnimation("Alpha");
TidalWavesBar.Flash.fadeOut:SetFromAlpha(0.25);
TidalWavesBar.Flash.fadeOut:SetToAlpha(1);
TidalWavesBar.Flash.fadeOut:SetDuration(0.33);
TidalWavesBar.Flash.fadeOut:SetEndDelay(0);

TidalWavesBar:SetScript("OnUpdate",function(self)
	if (Auras:CharacterCheck(3)) then
		local buff,_,_,count = UnitBuff('player',Auras:GetSpellName(53390));
		local color = Auras.db.char.layout[3].tidalWavesBar.emptyColor;
		local isAnimate = Auras.db.char.triggers.res.animTidalWaves;
		local isValidTarget = false;
	
		if (UnitAffectingCombat('player') or (UnitExists('target') and UnitIsFriend('player','target') and not UnitIsDead('target'))) then
			isValidTarget = true;
		else
			isValidTarget = false;
		end
		
		self:SetWidth(Auras.db.char.layout[3].tidalWavesBar.width);
		self:SetHeight(Auras.db.char.layout[3].tidalWavesBar.height);
		
		if (not count) then
			self:SetValue(2);
			self:SetStatusBarColor(color.r,color.g,color.b);
			
			if (isAnimate and isValidTarget) then
				if (not self.Flash:IsPlaying()) then
					self.Flash:Play();
				end
			else
				if (self.Flash:IsPlaying()) then
					self.Flash:Stop();
				end
				self:SetAlpha(1);
			end
		else
			self:SetStatusBarColor(0.35,0.76,1);
			self:SetValue(count or 0);
		end
		
		if (UnitAffectingCombat('player') or (UnitExists('target') and UnitIsFriend('player','target') and not UnitIsDead('target'))) then
			self:SetAlpha(1);
		else
			self:SetAlpha(0)
		end
	end
end);

SSA.TidalWavesBar = TidalWavesBar;
-- Build Cloudburst Totem Status Bar
--[[SSA.CloudburstAbsorbBar = CreateFrame("StatusBar","CloudburstAbsorbBar",AuraGroup);
local CloudburstAbsorbBar = SSA.CloudburstAbsorbBar;
CloudburstAbsorbBar:SetStatusBarTexture("Interface\\addons\\ShamanAuras\\media\\statusbar\\fifths");
CloudburstAbsorbBar:GetStatusBarTexture():SetHorizTile(false);
CloudburstAbsorbBar:GetStatusBarTexture():SetVertTile(false);
CloudburstAbsorbBar:SetPoint("CENTER",AuraGroup,"CENTER",0,-139);
CloudburstAbsorbBar:SetWidth(260);
CloudburstAbsorbBar:SetHeight(18);
CloudburstAbsorbBar:SetFrameStrata("LOW");
CloudburstAbsorbBar:SetStatusBarColor(0,0.5,1);
CloudburstAbsorbBar:Hide();

CloudburstAbsorbBar.bg = CloudburstAbsorbBar:CreateTexture(nil,"BACKGROUND");
CloudburstAbsorbBar.bg:SetTexture("Interface\\TargetingFrame\\UI-StatusBar");
CloudburstAbsorbBar.bg:SetAllPoints(true);
CloudburstAbsorbBar.bg:SetVertexColor(0,0,0);
CloudburstAbsorbBar.bg:SetAlpha(0.5);

CloudburstAbsorbBar.text = CloudburstAbsorbBar:CreateFontString(nil, "HIGH", "GameFontHighlightLarge");
CloudburstAbsorbBar.text:SetPoint("CENTER",CloudburstAbsorbBar,"RIGHT",-25,0);
CloudburstAbsorbBar.text:SetFont("Interface\\addons\\ShamanAuras\\Media\\fonts\\Continuum_Medium.ttf", 12);
CloudburstAbsorbBar.text:SetTextColor(1,1,1,1);

-- Build Cloudburst Timer Status Bar
CloudburstAbsorbBar.Timer = CreateFrame("StatusBar","CloudburstTimer",CloudburstAbsorbBar);
CloudburstAbsorbBar.Timer:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar");
CloudburstAbsorbBar.Timer:GetStatusBarTexture():SetHorizTile(false);
CloudburstAbsorbBar.Timer:GetStatusBarTexture():SetVertTile(false);
CloudburstAbsorbBar.Timer:SetWidth(CloudburstAbsorbBar:GetWidth());
CloudburstAbsorbBar.Timer:SetHeight(CloudburstAbsorbBar:GetHeight());
CloudburstAbsorbBar.Timer:SetFrameStrata("MEDIUM");
CloudburstAbsorbBar.Timer:SetPoint("CENTER",CloudburstAbsorbBar,"CENTER",0,0);
CloudburstAbsorbBar.Timer:SetStatusBarColor(1,1,1,0.35);
CloudburstAbsorbBar.Timer:Hide();

CloudburstAbsorbBar.Timer.text = CloudburstAbsorbBar.Timer:CreateFontString(nil, "HIGH", "GameFontHighlightLarge");
CloudburstAbsorbBar.Timer.text:SetPoint("CENTER",CloudburstAbsorbBar.Timer,"LEFT",25,0);
CloudburstAbsorbBar.Timer.text:SetFont("Interface\\addons\\ShamanAuras\\Media\\fonts\\Continuum_Medium.ttf", 12);
CloudburstAbsorbBar.Timer.text:SetTextColor(1,1,1,1);]]
--[[local BackdropCB = {
	bgFile    = "Interface\\AddOns\\rSetBackdrop\\tga\\background_flat",
	edgeFile  = "Interface\\AddOns\\rSetBackdrop\\tga\\UI-Tooltip-Border",
	tile      = false,
	tileSize  = 16, 
	edgeSize  = 16, 
	insets     = {
		left = 4,
		right = 4,
		top = 4,
		bottom = 4,
	},
};]]
local BackdropCB = SSA.BackdropCB;
local BackdropCBInner = {
	bgFile    = "Interface\\AddOns\\ShamanAuras\\media\\icons\\totems\\cloudburst_totem", 
	edgeFile  = "Interface\\AddOns\\ShamanAuras\\media\\textures\\tooltip_border",
	tile      = false,
	tileSize  = 16, 
	edgeSize  = 16,
}
--[[SSA.Cloudburst = CreateFrame("Frame","Cloudburst",AuraGroup);
_G["SSA_Cloudburst"] = SSA.Cloudburst;
local Cloudburst = SSA.Cloudburst;]]
local absorbed,duration,name
local Cloudburst = CreateFrame("Frame","Cloudburst",AuraGroup);
Cloudburst:SetFrameLevel(1);
--Cloudburst:SetWidth(150);
--Cloudburst:SetHeight(32);
Cloudburst:SetPoint("CENTER",AuraGroup,"CENTER",-200,0);
Cloudburst:SetBackdrop(BackdropCB);
Cloudburst:SetBackdropColor(0.15,0.15,0.15,0.6);
Cloudburst:SetBackdropBorderColor(1,1,1,1);
Cloudburst:SetAlpha(0);
Cloudburst:Show();

Cloudburst.text = Cloudburst:CreateFontString(nil, "MEDIUM", "GameFontHighlightLarge");
Cloudburst.text:SetPoint("RIGHT",Cloudburst,"RIGHT",-5,-1);
--Cloudburst.text:SetPoint("CENTER",0,0);
Cloudburst.text:SetTextColor(0,1,0,1);
Cloudburst.text:SetFont("Fonts\\FRIZQT__.TTF",22,"OUTLINE");
Cloudburst.text:SetJustifyH("LEFT");
Cloudburst.text:SetText('0');
--[[Cloudburst.inner = CreateFrame("Frame",nil,Cloudburst);
Cloudburst.inner:SetPoint("TOPLEFT",Cloudburst,"TOPLEFT",8,-8);
Cloudburst.inner:SetPoint("BOTTOMRIGHT",Cloudburst,"BOTTOMRIGHT",-8,8);
Cloudburst.inner:SetBackdrop(BackdropCBInner);
Cloudburst.inner:SetBackdropColor(0.15,0.8,1,0);
Cloudburst.inner:SetBackdropBorderColor(1,1,1,1);]]

Cloudburst.icon = CreateFrame("Frame",nil,Cloudburst);
Cloudburst:SetFrameLevel(2);
Cloudburst.icon:SetWidth(40);
Cloudburst.icon:SetHeight(40);
Cloudburst.icon:SetPoint("LEFT",Cloudburst,"LEFT",0,0);
Cloudburst.icon:SetBackdrop(BackdropCB);
Cloudburst.icon:SetBackdropColor(1,1,1,0);
Cloudburst.icon:SetBackdropBorderColor(1,1,1,1);

Cloudburst.icon.texture = Cloudburst.icon:CreateTexture(nil,"BACKGROUND");
Cloudburst.icon.texture:SetTexture("Interface\\addons\\ShamanAuras\\Media\\icons\\totems\\cloudburst_totem_bevel");
Cloudburst.icon.texture:SetAllPoints(Cloudburst.icon);

Cloudburst.icon.text = Cloudburst.icon:CreateFontString(nil, "MEDIUM", "GameFontHighlightLarge");
Cloudburst.icon.text:SetAllPoints(Cloudburst.icon);
Cloudburst.icon.text:SetPoint("CENTER",0,0);
Cloudburst.icon.text:SetTextColor(1,1,0,1);
Cloudburst.icon.text:SetFont("Interface\\addons\\ShamanAuras\\media\\fonts\\PT_Sans_Narrow.TTF", 16,"OUTLINE");
	
Cloudburst:SetScript("OnUpdate",function(self,elapsed)
	if (Auras:CharacterCheck(3)) then
		_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,_,absorbed = UnitBuff('player',Auras:GetSpellName(157153));
	
		for i=1,5 do
			_,name = GetTotemInfo(i);
			if (name == "Cloudburst Totem") then
				duration = GetTotemTimeLeft(i);
			end
		end

		if (absorbed and Auras.db.char.aura[3].CloudburstAbsorbBar) then
			self.icon.text:SetText(duration);
			self:SetAlpha(1);
			self.text:SetText(absorbed);
		else
			self:SetAlpha(0);
			self.text:SetText('0');
			SSA.CloudburstTotemBar:Hide();
		end
	end
end);
_G["SSA_Cloudburst"] = Cloudburst;
SSA.Cloudburst = Cloudburst;
--[[Cloudburst.icon.inner = CreateFrame("Frame",nil,Cloudburst);
Cloudburst.icon.inner:SetPoint("TOPLEFT",Cloudburst,"TOPLEFT",8,-8);
Cloudburst.icon.inner:SetPoint("BOTTOMRIGHT",Cloudburst,"BOTTOMRIGHT",-8,8);
Cloudburst.icon.inner:SetBackdrop(BackdropCBInner);
Cloudburst.icon.inner:SetBackdropColor(0.15,0.8,1,0);
Cloudburst.icon.inner:SetBackdropBorderColor(1,1,1,1);]]

-- Build Earthen Shield Status Bar
--SSA.EarthenShieldTotemBar = CreateFrame("StatusBar","EarthenShieldTotemBar",AuraGroup);
--local EarthenShieldTotemBar = SSA.EarthenShieldTotemBar;


local EarthenShieldTotemBar = CreateFrame("StatusBar","EarthenShieldTotemBar",AuraGroup);
_G["SSA_EarthenShield"] = EarthenShieldTotemBar;
EarthenShieldTotemBar:SetStatusBarTexture("Interface\\addons\\ShamanAuras\\media\\statusbar\\fifths");
EarthenShieldTotemBar:GetStatusBarTexture():SetHorizTile(false);
EarthenShieldTotemBar:GetStatusBarTexture():SetVertTile(false);
EarthenShieldTotemBar:SetPoint("CENTER",AuraGroup,"CENTER",0,-139);
--EarthenShieldTotemBar:SetWidth(260);
--EarthenShieldTotemBar:SetHeight(18);
EarthenShieldTotemBar:SetFrameStrata("LOW");
EarthenShieldTotemBar:SetStatusBarColor(0.91,0.41,1);
EarthenShieldTotemBar:Show();
EarthenShieldTotemBar:SetAlpha(0);

EarthenShieldTotemBar.bg = EarthenShieldTotemBar:CreateTexture(nil,"BACKGROUND");
EarthenShieldTotemBar.bg:SetTexture("Interface\\TargetingFrame\\UI-StatusBar");
EarthenShieldTotemBar.bg:SetAllPoints(true);
EarthenShieldTotemBar.bg:SetVertexColor(0,0,0);
EarthenShieldTotemBar.bg:SetAlpha(0.5);

EarthenShieldTotemBar.text = EarthenShieldTotemBar:CreateFontString(nil, "HIGH", "GameFontHighlightLarge");
EarthenShieldTotemBar.text:SetPoint("CENTER",EarthenShieldTotemBar,"RIGHT",-25,0);
EarthenShieldTotemBar.text:SetFont("Interface\\addons\\ShamanAuras\\Media\\fonts\\Continuum_Medium.ttf", 12);
EarthenShieldTotemBar.text:SetTextColor(1,1,1,1);

-- Build Earthen Shield Timer Status Bar
EarthenShieldTotemBar.Timer = CreateFrame("StatusBar","EarthenShieldTimer",EarthenShieldTotemBar);
EarthenShieldTotemBar.Timer:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar");
EarthenShieldTotemBar.Timer:GetStatusBarTexture():SetHorizTile(false);
EarthenShieldTotemBar.Timer:GetStatusBarTexture():SetVertTile(false);
--EarthenShieldTotemBar.Timer:SetWidth(EarthenShieldTotemBar:GetWidth());
--EarthenShieldTotemBar.Timer:SetHeight(EarthenShieldTotemBar:GetHeight());
EarthenShieldTotemBar.Timer:SetFrameStrata("MEDIUM");
EarthenShieldTotemBar.Timer:SetPoint("CENTER",EarthenShieldTotemBar,"CENTER",0,0);
EarthenShieldTotemBar.Timer:SetStatusBarColor(1,1,1,0.35);
EarthenShieldTotemBar.Timer:SetMinMaxValues(0,15);
EarthenShieldTotemBar.Timer:Show();
EarthenShieldTotemBar.Timer:SetAlpha(0);

EarthenShieldTotemBar.Timer.text = EarthenShieldTotemBar.Timer:CreateFontString(nil, "HIGH", "GameFontHighlightLarge");
EarthenShieldTotemBar.Timer.text:SetPoint("CENTER",EarthenShieldTotemBar.Timer,"LEFT",25,0);
EarthenShieldTotemBar.Timer.text:SetFont("Interface\\addons\\ShamanAuras\\Media\\fonts\\Continuum_Medium.ttf", 12);
EarthenShieldTotemBar.Timer.text:SetTextColor(1,1,1,1);

EarthenShieldTotemBar:SetScript("OnUpdate",function(self,elapsed)
	local timer,seconds = Auras:parseTime(((EST_Expires or 0) - GetTime()),true);
	
	if (self:GetWidth() ~= Auras.db.char.layout[3].earthenShieldBar.width) then
		self:SetWidth(Auras.db.char.layout[3].earthenShieldBar.width);
		self.Timer:SetWidth(self:GetWidth());
	end
	
	if (self:GetHeight() ~= Auras.db.char.layout[3].earthenShieldBar.height) then
		self:SetHeight(Auras.db.char.layout[3].earthenShieldBar.height);
		self.Timer:SetHeight(self:GetHeight());
	end
		
	if ((seconds or 0) > 0.1) then
		self.Timer:SetValue(seconds);
		self.Timer.text:SetText(timer);
	else
		self.Timer:SetValue(0);
		self.Timer.text:SetText('');
	end
end);
--[[EarthenShieldTotemBar:SetScript("OnUpdate",function(self)
	local buff,_,_,count,_,_,expires = UnitBuff('player',Auras:GetSpellName(210714));

	if (buff) then
		local timer,seconds = Auras:parseTime(expires - GetTime(),false);
		
		self:SetAlpha(1);
		self:SetValue(count);
		self.Timer:SetValue(seconds);
		self.Timer.text:SetText(string.format("%.1f",seconds));
	else
		self:SetAlpha(0);
	end
end)]]


SSA.EarthenShieldTotemBar = EarthenShieldTotemBar;

--[[local function UpdateEarthenShield()
	local progress = ((Auras.db.char.info.totems.eShield.dmg or 0) / Auras.db.char.info.totems.eShield.hp * 100);
	progress = 100 - progress;
	local remains = (Auras.db.char.info.totems.eShield.hp - (Auras.db.char.info.totems.eShield.dmg or 0));
	
	if (remains > 0) then
		SSA.EarthenShieldTotemBar:SetValue(remains);
		SSA.EarthenShieldTotemBar.text:SetText(tostring(math.ceil(progress)).."%");
	else
		SSA.EarthenShieldTotemBar:SetAlpha(0);
		--SSA.EarthenShieldTotemBar.Timer:Hide();
	end
end]]

-- Build Buff Timer Status Bar Group
SSA.BuffTimerBarGrpRes = CreateFrame("Frame","BuffTimerBarGrpRes",AuraGroup);
local BuffTimerBarGrp = SSA.BuffTimerBarGrpRes;
BuffTimerBarGrp:SetWidth(131);
BuffTimerBarGrp:SetHeight(180);
BuffTimerBarGrp:SetBackdrop(nil);
BuffTimerBarGrp:Show();

-- Build Main Timer Status Bar Group
SSA.MainTimerBarGrpRes = CreateFrame("Frame","MainTimerBarGrpRes",AuraGroup);
local MainTimerBarGrp = SSA.MainTimerBarGrpRes;
MainTimerBarGrp:SetWidth(105);
MainTimerBarGrp:SetHeight(180);
MainTimerBarGrp:SetBackdrop(nil);
MainTimerBarGrp:Show();

-- Build Utility Timer Status Bar Group
SSA.UtilTimerBarGrpRes = CreateFrame("Frame","UtilTimerBarGrpRes",AuraGroup);
local UtilTimerBarGrp = SSA.UtilTimerBarGrpRes;
UtilTimerBarGrp:SetWidth(47);
UtilTimerBarGrp:SetHeight(180);
UtilTimerBarGrp:SetBackdrop(nil);
UtilTimerBarGrp:Show();

-- Build Restoration Vertical Status Bars
SSA.AncestralGuidanceBarRes = CreateFrame("StatusBar","AncestralGuidanceBarRes",BuffTimerBarGrp);
SSA.AncestralProtectionTotemBar = CreateFrame("StatusBar","AncestralProtectionTotemBar",MainTimerBarGrp);
SSA.AscendanceBarRes = CreateFrame("StatusBar","AscendanceBarRes",BuffTimerBarGrp);
SSA.AstralShiftBarRes = CreateFrame("StatusBar","AstralShiftBarRes",BuffTimerBarGrp);
SSA.BloodlustBarRes = CreateFrame("StatusBar","BloodlustBarRes",BuffTimerBarGrp);
SSA.CloudburstTotemBar = CreateFrame("StatusBar","CloudburstTotemBar",MainTimerBarGrp);
SSA.EarthgrabTotemBarRes = CreateFrame("StatusBar","EarthgrabTotemBarRes",UtilTimerBarGrp);
SSA.HealingStreamTotemBarOne = CreateFrame("StatusBar","HealingStreamTotemBarOne",MainTimerBarGrp);
SSA.HealingStreamTotemBarTwo = CreateFrame("StatusBar","HealingStreamTotemBarTwo",MainTimerBarGrp);
SSA.HealingTideTotemBar = CreateFrame("StatusBar","HealingTideTotemBar",MainTimerBarGrp);
SSA.HeroismBarRes = CreateFrame("StatusBar","HeroismBarRes",BuffTimerBarGrp);
SSA.HexBarRes = CreateFrame("StatusBar","HexBarRes",UtilTimerBarGrp);
SSA.SpiritLinkTotemBar = CreateFrame("StatusBar","SpiritLinkTotemBar",MainTimerBarGrp);
SSA.SpiritwalkersGraceBar = CreateFrame("StatusBar","SpiritwalkersGraceBar",BuffTimerBarGrp);
SSA.TimeWarpBarRes = CreateFrame("StatusBar","TimeWarpBarRes",BuffTimerBarGrp);
SSA.UnleashLifeBar = CreateFrame("StatusBar","UnleashLifeBar",BuffTimerBarGrp);
SSA.VoodooTotemBarRes = CreateFrame("StatusBar","VoodooTotemBarRes",UtilTimerBarGrp);
SSA.WindRushTotemBarRes = CreateFrame("StatusBar","WindRushTotemBarRes",UtilTimerBarGrp);

local buffIDs = {
	[108281] = SSA.AncestralGuidanceBarRes,
	[114052] = SSA.AscendanceBarRes,
	[108271] = SSA.AstralShiftBarRes,
	[2825]   = SSA.BloodlustBarRes,
	[32182]  = SSA.HeroismBarRes,
	[79206]  = SSA.SpiritwalkersGraceBar,
	[80353]  = SSA.TimeWarBarRes,
	[73685]  = SSA.UnleashLifeBar,
}
local mainIDs = {
	["Ancestral Protection Totem"] = SSA.AncestralProtTotemBar,
	["Cloudburst Totem"] = SSA.CloudburstTotemBar,
	["Healing Stream Totem1"] = SSA.HealingStreamTotemBarOne,
	["Healing Stream Totem2"] = SSA.HealingStreamTotemBarTwo,
	["Healing Tide Totem"] = SSA.HealingTideTotemBar,
	["Spirit Link Totem"]  = SSA.SpiritLinkTotemBar,
}
local utilIDs = {
	["Earthgrab Totem"]  = SSA.EarthgrabTotemBarRes,
	["Voodoo Totem"] = SSA.VoodooTotemBarRes,
	["Wind Rush Totem"] = SSA.WindRushTotemBarRes,
}

local xPos = {
	[1] = 5,
	[2] = 25,
	[3] = 45,
	[4] = 65,
	[5] = 85,
	[6] = 105,
	[7] = 125,
	[8] = 145,
	[9] = 165,
	[10] = 185,
}

local xOffset = {
	[1] = -33,
	[2] = -53,
	[3] = -73,
	[4] = -93,
	[5] = -113,
	[6] = -133,
	[7] = -153,
	[8] = -173,
	[9] = -193,
}

BuffTimerBarGrp:SetScript("OnUpdate",function(self,event,...)
	if (Auras:CharacterCheck(3)) then
		for i=1,getn(buffTable) do
			local buff,_,_,_,_,duration,expires = UnitBuff('player',Auras:GetSpellName(buffTable[i]));
			local timer,seconds = Auras:parseTime(expires - GetTime(),true);
			
			buffIDs[buffTable[i]]:SetMinMaxValues(0,duration);
			buffIDs[buffTable[i]]:SetValue(seconds);
			buffIDs[buffTable[i]].timetext:SetText(timer);
			buffIDs[buffTable[i]]:SetPoint("LEFT",xPos[i],0);
			buffIDs[buffTable[i]]:Show();
		end
	end
end);

MainTimerBarGrp:SetScript("OnUpdate",function(self,event,...)
	if (Auras:CharacterCheck(3)) then
		local totemCtr = 0
		local streamCtr = 0;
		local x
		local _,_,_,selected = GetTalentInfo(6,3,1);
	
		for i=1,5 do
			local _,name,start,duration = GetTotemInfo(i);

			if (duration > 0) then
				local timer,seconds = Auras:parseTime((start + duration) - GetTime(),true);

				if (name == "Healing Stream Totem" and not selected) then
					name = "Healing Stream Totem1";
				elseif (name == "Healing Stream Totem" and selected) then
					streamCtr = streamCtr + 1;
					name = "Healing Stream Totem"..streamCtr;
				end

				if (mainIDs[name]) then
					totemCtr = totemCtr + 1;
				
					if (seconds > 0.1) then
						mainIDs[name]:SetMinMaxValues(0,duration);
						mainIDs[name]:SetValue(seconds);
						mainIDs[name].timetext:SetText(timer);
						mainIDs[name]:SetPoint("RIGHT",(xPos[totemCtr] * -1),0);
						mainIDs[name]:Show();
					else
						totemCtr = totemCtr - 1;
						mainIDs[name]:SetValue(0);
						mainIDs[name]:Hide();
					end
				end
			end
		end
		mainTotems = totemCtr;
	end
end);

UtilTimerBarGrp:SetScript("OnUpdate",function(self,event,...)
	if (Auras:CharacterCheck(3)) then
		local totemCtr = 0
		local x
	
		for i=1,5 do
			local _,name,start,duration = GetTotemInfo(i);
			
			if (duration > 0) then
				local timer,seconds = Auras:parseTime((start + duration) - GetTime(),true);
				
				if (utilIDs[name]) then
					totemCtr = totemCtr + 1;
					
					if (mainTotems > 0) then
						x = xOffset[mainTotems + (totemCtr - 1)];
					else
						x = (xPos[totemCtr] * -1);
					end
				
					if (seconds > 0.1) then
						utilIDs[name]:SetMinMaxValues(0,duration);
						utilIDs[name]:SetValue(seconds);
						utilIDs[name].timetext:SetText(timer);
						utilIDs[name]:SetPoint("RIGHT",x,0);
						utilIDs[name]:Show();
					else
						totemCtr = totemCtr - 1;
						utilIDs[name]:SetValue(0);
						utilIDs[name]:Hide();
					end
				end
			end
		end
	end
end);

--[[-- Undulation Notification
SSA.Undulation = CreateFrame("Frame","Undulation",AuraGroup);
local Undulation = SSA.Undulation;
Undulation:SetPoint("CENTER",0,-210);
Undulation:SetWidth(400);
Undulation:SetHeight(180);
Undulation:SetFrameStrata("BACKGROUND");
Undulation:Hide();

Undulation.texture = Undulation:CreateTexture(nil,"LOW");
Undulation.texture:SetTexture("Textures\\SpellActivationOverlays\\Monk_Serpent");
Undulation.texture:SetPoint("CENTER",Undulation,"CENTER",0,0);
Undulation.texture:SetWidth(Undulation:GetWidth() + 20);
Undulation.texture:SetHeight(Undulation:GetHeight());
Undulation.texture:SetRotation(math.rad(-90));
Undulation.texture:SetAlpha(0.7);

-- Animation for Totem Mastery Notification Texture
Undulation.Flash = Undulation.texture:CreateAnimationGroup();
Undulation.Flash:SetLooping("BOUNCE");

Undulation.Flash.fadeIn = Undulation.Flash:CreateAnimation("Alpha");
Undulation.Flash.fadeIn:SetFromAlpha(1);
Undulation.Flash.fadeIn:SetToAlpha(0.4);
Undulation.Flash.fadeIn:SetDuration(0.4);

Undulation.Flash.fadeOut = Undulation.Flash:CreateAnimation("Alpha");
Undulation.Flash.fadeOut:SetFromAlpha(0.4);
Undulation.Flash.fadeOut:SetToAlpha(1);
Undulation.Flash.fadeOut:SetDuration(0.4);
Undulation.Flash.fadeOut:SetEndDelay(0);]]

--SSA.ErrorFrame.text:SetText("Name: "..StormkeeperCharges:GetName())
local buff
local Undulation = CreateFrame("Frame","Undulation",AuraGroup);
--local StormkeeperCharge1 = SSA.StormkeeperCharge1;
Undulation:SetWidth(60);
Undulation:SetHeight(60);
Undulation:SetPoint("CENTER",AuraGroup,"CENTER",0,-275);
Undulation:SetAlpha(0);
Undulation:Show();

Undulation.Orb = CreateFrame("PlayerModel","StormkeeperCharge1_Orb",Undulation);
Undulation.Orb:SetModel("SPELLS/Monk_ForceSpere_Orb.m2");
Undulation.Orb:SetFrameStrata(low);
Undulation.Orb:SetAllPoints(Undulation);
--Undulation.Orb:SetCamera(1);
Undulation.Orb:SetPosition(0,0,0);
Undulation.Orb:SetAlpha(1);
--Undulation.Orb:SetModelScale(0.021);
Undulation.Orb:SetSequence(1);
Undulation.Orb:SetRotation(0);
Undulation:SetScript("OnUpdate",function(self)
	buff = UnitBuff('player',Auras:GetSpellName(216251));
	
	if (buff and Auras.db.char.aura[3].Undulation) then
		self:SetAlpha(1);
	else
		self:SetAlpha(0);
	end
end);
SSA.Undulation = Undulation;
_G["SSA_Undulation"] = Undulation;

EventFrame:SetScript("OnEvent",function(self,event,...)
	if (Auras:CharacterCheck(3)) then
		local _,subevent,_,sGUID,source,_,_,petGUID,name,_,_,spellID,_,_,damage = ...

		if (event ~= "COMBAT_LOG_EVENT_UNFILTERED") then
			return
		end
		--SSA.ErrorFrame.text:SetText("SubEvent: "..subevent.."\n");
		--SSA.ErrorFrame.text:SetText(Auras:CurText('ErrorFrame').."Spell ID: "..spellID.."\n");
		if (sGUID == UnitGUID('player')) then
			if (subevent == "SPELL_AURA_APPLIED") then
				if (buffIDs[spellID]) then
					table.insert(buffTable,spellID);
				end
			elseif (subevent == "SPELL_AURA_REMOVED") then
				if (buffIDs[spellID]) then
					for i=1,getn(buffTable) do
						if (buffTable[i] == spellID) then
							table.remove(buffTable,i)
						end
					end
					buffIDs[spellID]:Hide();
				end
			elseif (subevent == "SPELL_SUMMON" and spellID == 198838) then
				SSA.ErrorFrame.text:SetText("EST Summoned");
				local EST = SSA.EarthenShieldTotemBar;
				EST_Expires = GetTime() + 15;
				SSA.EarthenShieldTotemBar:SetAlpha(1);
				SSA.EarthenShieldTotemBar.Timer:SetAlpha(1);
				SSA.EarthenShieldTotemBar:SetMinMaxValues(0,UnitHealthMax('player'));
				SSA.EarthenShieldTotemBar.text:SetText("100%");
				SSA.EarthenShieldTotemBar:SetValue(UnitHealthMax('player'));
				Auras.db.char.info.totems.eShield.hp = UnitHealthMax('player');
				Auras.db.char.info.totems.eShield.dmg = 0;
				EST_GUID = petGUID;
			elseif ((subevent == "SPELL_DAMAGE" or subevent == "SWING_DAMAGE") and name == L["Earthen Shield Totem"]) then
				Auras.db.char.info.totems.eShield.dmg = (Auras.db.char.info.totems.eShield.dmg or 0) + damage;
				Auras:UpdateEarthenShield(EarthenShieldTotemBar);
			elseif (subevent == "UNIT_DIED" and name == L["Earthen Shield Totem"]) then
				SSA.ErrorFrame.text:SetText("DEAD");
				
			end
		end
		if (subevent == "UNIT_DIED" and EST_GUID == petGUID) then
			EarthenShieldTotemBar:SetAlpha(0);
			EarthenShieldTotemBar.Timer:SetAlpha(0);
			EST_GUID = nil;
		end
	end
end);

-------------------------------------------------------------------------------------------------------
----- Build Move UI
-------------------------------------------------------------------------------------------------------

SSA.MoveRes = CreateFrame("Frame","MoveRes",UIParent);

SSA.AuraObjectsRes = {
	[1] = {
		alpha = nil,
		object = AuraGroup,
		backdrop = "BackdropSB",
	},
	[2] = {
		alpha = nil,
		object = LargeIconGrpTop,
		backdrop = "BackdropSB",
	},
	[3] = {
		alpha = nil,
		object = LargeIconGrpBot,
		backdrop = "BackdropSB",
	},
	[4] = {
		alpha = nil,
		object = SmallIconGrpLeft,
		backdrop = "BackdropSB",
	},
	[5] = {
		alpha = nil,
		object = SmallIconGrpRight,
		backdrop = "BackdropSB",
	},
	[6] = {
		alpha = nil,
		object = BuffTimerBarGrp,
		backdrop = "BackdropSB",
	},
	[7] = {
		alpha = nil,
		object = MainTimerBarGrp,
		backdrop = "BackdropSB",
	},
	[8] = {
		alpha = nil,
		object = UtilTimerBarGrp,
		backdrop = "BackdropSB",
	},
	[9] = {
		alpha = false,
		object = EarthenShieldTotemBar,
		statusbar = {
			r = 0.91,
			g = 0.41,
			b = 1,
			m = 'player',
		},
	},
	[10] = {
		alpha = false,
		object = TidalWavesBar,
		statusbar = {
			r = 0.35,
			g = 0.76,
			b = 1,
			m = 2,
		},
	},
	--[11] = LavaSurge,
	[11] = {
		alpha = true,
		model = {
			[1] = Undulation.Orb,
		},
		object = Undulation,
		backdrop = 'BackdropSB',
	},
	[12] = {
		object = Cloudburst,
		alpha = true,
	},
	--[[[11] = CloudburstAbsorbBar,
	[12] = LavaSurge,]]
}

SSA.MoveStringsRes = {
	[1] = L["Move All Auras"],
	[2] = L["Move Top Auras"],
	[3] = L["Move Bottom Auras"],
	[4] = L["Move Left Auras"],
	[5] = L["Move Right Auras"],
	[6] = L["Move Buff Timer Bars"],
	[7] = L["Move Main Timer Bars"],
	[8] = L["Move Utility Timer Bars"],
	[9] = L["Move Earthen Shield Bar"],
	[10] = L["Move Tidal Waves Bar"],
	--[11] = L["Move Lava Surge Texture"],
	[11] = L["Move Undulation"],
	[12] = L["Cloudburst Healing Stored"],
	--[12] = L["Move Lava Surge Texture"],
}
